#
# spec file for package parsec
#
# Copyright (c) 2024 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global rustflags '-Clink-arg=-Wl,-z,relro,-z,now'
%define archive_version 1.4.1

%{?systemd_ordering}
Name:           parsec
Version:        1.4.1
Release:        1.2
Summary:        Platform AbstRaction for SECurity
License:        Apache-2.0
URL:            https://parallaxsecond.github.io/parsec-book
Source0:        https://github.com/parallaxsecond/parsec/archive/%{archive_version}.tar.gz#/parsec-%{archive_version}.tar.gz
Source1:        vendor.tar.xz
Source3:        parsec.service
Source4:        config.toml
Source5:        parsec.conf
Source6:        system-user-parsec.conf
Source10:       https://git.trustedfirmware.org/TS/trusted-services.git/snapshot/trusted-services-389b506.tar.gz
Patch1:         0001-Fix-unnecessary-qualifications-error.patch
BuildRequires:  cargo >= 1.66
BuildRequires:  clang-devel
BuildRequires:  cmake
BuildRequires:  gcc-c++
BuildRequires:  llvm-devel
%if 0%{?suse_version} == 1500
# Fix build with GCC13 on Backports SLE15-SPx - Avoid to get -lstdc++ not found
BuildRequires:  libstdc++6-devel-gcc13
%endif
BuildRequires:  cargo-packaging
BuildRequires:  pkgconfig
BuildRequires:  protobuf-devel
BuildRequires:  python3
# jwt-svid-authenticator (SPIFFE-based authenticator) needs rust >= 1.53
BuildRequires:  rust >= 1.53
BuildRequires:  sysuser-tools
BuildRequires:  pkgconfig(tss2-esys) >= 2.3.3
# opensc is used to initialize HSM keys (PKCS#11 backend)
Recommends:     opensc
# /dev/tpm* are owned by tss user
Requires(pre):  system-user-tss
# tpm2-0-tss holds the udev rule to make /dev/tpm* owned by tss user
Requires:       tpm2-0-tss
# Without libtss2-tcti-device0 parsec fails to start TPM properly
Requires:       libtss2-tcti-device0
ExcludeArch:    %{ix86} armv6l armv6hl

%description
PARSEC is the Platform AbstRaction for SECurity, an open-source initiative to provide
a common API to hardware security and cryptographic services in a platform-agnostic way.
This abstraction layer keeps workloads decoupled from physical platform details,
enabling cloud-native delivery flows within the data center and at the edge.

%package -n system-user-parsec
Summary:        System user and group parsec
%sysusers_requires

%description -n system-user-parsec
Package to install system user 'parsec'

%prep
%setup -q -a1 -a10 -n parsec-%{archive_version}
%autopatch -p1
rmdir trusted-services-vendor
mv trusted-services-389b506 trusted-services-vendor
# Enable all providers
sed -i -e 's#default = \["unix-peer-credentials-authenticator"\]##' Cargo.toml
# Features available in 1.2.0-rc1:
# all-providers = ["tpm-provider", "pkcs11-provider", "mbed-crypto-provider", "trusted-service-provider"]
# all-authenticators = ["direct-authenticator", "unix-peer-credentials-authenticator", "jwt-svid-authenticator"]
%if 0%{?suse_version} > 1550
# But disable "jwt-svid-authenticator"/SPIFFE with gcc13 until build fixed upstream - https://github.com/parallaxsecond/parsec/issues/672
sed -i -e 's#all-authenticators = \["direct-authenticator", "unix-peer-credentials-authenticator", "jwt-svid-authenticator"\]#all-authenticators = \["direct-authenticator", "unix-peer-credentials-authenticator"\]#' Cargo.toml
%endif
# But disable "trusted-service-provider" until we have a trusted-services package
echo 'default = ["tpm-provider", "pkcs11-provider", "mbed-crypto-provider", "all-authenticators"]' >> Cargo.toml

%build
export PROTOC=%{_bindir}/protoc
export PROTOC_INCLUDE=%{_includedir}
%cargo_build
%sysusers_generate_pre %{SOURCE6} parsec

%install
export PROTOC=%{_bindir}/protoc
export PROTOC_INCLUDE=%{_includedir}
%cargo_install
mkdir -p %{buildroot}%{_sysusersdir}
install -m 0644 %{SOURCE6} %{buildroot}%{_sysusersdir}/
install -D -p -m0644 %{SOURCE3} %{buildroot}%{_unitdir}/parsec.service
install -D -p -m0644 %{SOURCE4} %{buildroot}%{_sysconfdir}/parsec/config.toml
install -D -p -m0644 %{SOURCE5} %{buildroot}%{_tmpfilesdir}/parsec.conf
install -d -m0755 %{buildroot}%{_localstatedir}/lib/parsec
# Move parsec to _libexecdir
mkdir -p %{buildroot}%{_libexecdir}
mv target/release/parsec %{buildroot}%{_libexecdir}

# currently delete it from bin if there
rm -f %{buildroot}%{_bindir}/parsec
# Clean-up
find %{buildroot} -name .crates2.json -delete
rm -rf %{buildroot}%{_datadir}/cargo/registry

%check
export PROTOC=%{_bindir}/protoc
export PROTOC_INCLUDE=%{_includedir}
%cargo_test -- --lib

%pre -n system-user-parsec -f parsec.pre
%service_add_pre parsec.service

%post
%service_add_post parsec.service
%tmpfiles_create %_tmpfilesdir/parsec.conf

%preun
%service_del_preun parsec.service

%postun
%service_del_postun parsec.service

%files
%license LICENSE
%doc README.md config.toml
%attr(0750,parsec,parsec) %dir %{_sysconfdir}/parsec/
%attr(0750,parsec,parsec) %dir %{_localstatedir}/lib/parsec/
%attr(0644,parsec,parsec) %config(noreplace) %{_sysconfdir}/parsec/config.toml
%{_libexecdir}/parsec
%{_tmpfilesdir}/parsec.conf
%{_unitdir}/parsec.service

%files -n system-user-parsec
%{_sysusersdir}/system-user-parsec.conf

%changelog
* Sat Nov 30 2024 Rein Fernhout <me@levitati.ng>
- Do not add lock group to parsec
- Added a patch from upstream to fix a build error:
  * 0001-Fix-unnecessary-qualifications-error.patch
- Update to 1.4.1:
  * Full changelog: https://github.com/parallaxsecond/parsec/compare/1.4.0...1.4.1
* Tue Apr 16 2024 Guillaume GARDET <guillaume.gardet@opensuse.org>
-  Update to 1.4.0:
  * Full changelog: https://github.com/parallaxsecond/parsec/compare/1.4.0-rc2...1.4.0
- Simplify libstdc++-devel handling for Leap 15.x
* Tue Apr 16 2024 Max Lin <mlin@suse.com>
- Fix build on Leap 15.6
  * installed the corresponding libstdc++-devel to avoid the -lstdc++ not found
* Thu Mar 28 2024 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.4.0-rc2:
  * Full changelog: https://github.com/parallaxsecond/parsec/compare/1.3.0...1.4.0-rc2
* Mon Oct 30 2023 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.3.0:
  * Full changelog:  https://github.com/parallaxsecond/parsec/compare/1.3.0-rc1...1.3.0
* Tue Oct 17 2023 Guillaume <guillaume@arm.com>
- Fix build on Leap 15.5
- Update to 1.3.0-rc1:
  * Changelog: https://github.com/parallaxsecond/parsec/compare/1.2.0...1.3.0-rc1
* Thu Apr 20 2023 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Fix build on Leap 15.5
- Update to 1.2.0:
  * Closed issue since 1.2.0-rc1:
  - Parsec 1.1 fails to build with meta-security master branch
* Mon Mar 27 2023 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Disable jwt-svid-authenticator (SPIFFE) until fixed upstream
  with gcc13 - https://github.com/parallaxsecond/parsec/issues/672
* Thu Mar 23 2023 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.2.0-rc1
- Drop upstream patch:
  * 664.patch
* Wed Mar 15 2023 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Add patch to fix build on Tumbleweed (update tss-esapi to 7.2.0):
  * 664.patch
- Add <param name="update">true</param> to _service to apply
  security updates
- Use cargo-packaging for all flavors
- Enable cargo_audit
* Thu Jan 19 2023 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Add libstdc++6-devel-gcc11 to fix build with GCC11 on
  Backports SLE15-SP4/5
* Tue Oct  4 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.1.0:
  * Main updates:
  - Add generate random support into TPM and PKCS11 providers
  - Implement configurable exclusion of deprecated primitives
  - Allow binary PIN values for PKCS11 provider
  - Recognise a PKCS11 hardware token with its serial number
    instead of slot number
  * Full changelog: https://github.com/parallaxsecond/parsec/releases/tag/1.1.0
* Mon Sep 26 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.1.0-rc2
* Wed Apr  6 2022 Marcus Meissner <meissner@suse.com>
- rust-packaging -> cargo-packaging (bsc#1198131)
* Thu Mar 31 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.0.0:
  * Update changelog file only
- Disable build on x86 since the build fails
* Mon Mar 21 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.0.0-rc3:
  * Changelog: https://github.com/parallaxsecond/parsec/compare/1.0.0-rc2...1.0.0-rc3
  * Fixes CVE-2022-24713 - boo#1196972
* Tue Mar  8 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.0.0-rc2:
  * Changelog: https://github.com/parallaxsecond/parsec/compare/1.0.0-rc1...1.0.0-rc2
- Remove CryptoAuthLib (CAL) provider as it is unmaintained.
* Thu Feb 24 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.0.0-rc1:
  * Changelog: https://github.com/parallaxsecond/parsec/compare/0.8.1...1.0.0-rc1
- Drop upstream patch:
  * harden_parsec.service.patch
* Thu Jan 13 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 0.8.1:
  * Changelog: https://github.com/parallaxsecond/parsec/compare/0.8.0...0.8.1
* Thu Dec  9 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Fix /run/parsec permission to 755. This is enough for all
  users to access the service - boo#1193484 - CVE-2021-36781
* Fri Oct 15 2021 Johannes Segitz <jsegitz@suse.com>
- Added hardening to systemd service(s) (bsc#1181400). Added patch(es):
  * harden_parsec.service.patch
  Modified:
  * parsec.service
  * Upstream submission: https://github.com/parallaxsecond/parsec/issues/569
* Mon Sep 27 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Allow access to /run/parsec to all users
- Requires rust 1.53+ (now also available in 15.3/15-SP3 via Update)
  which allow to enable same features accross distros
* Fri Aug  6 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 0.8.0:
  * Changelog: https://github.com/parallaxsecond/parsec/compare/0.7.2...0.8.0
- Drop upstream patch:
  * parsec-fix-old-rust.patch
- Disable 'trusted-service-provider' as it currently fails to build
- Disable 'jwt-svid-authenticator' (SPIFFE-based authenticator)
  on Leap, as it cannot be compiled with rust 1.43.1
* Fri Apr 30 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Fix build with old rust used in Leap 15.3/SLE15-SP3.
  * parsec-fix-old-rust.patch
* Wed Apr 14 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Requires 'tpm2-0-tss' as it holds the udev rule to make /dev/tpm*
  owned by tss user
- Requires 'libtss2-tcti-device0' as it is required to start parsec
  with TPM support
* Wed Apr  7 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Enable all authenticators which adds 'direct-authenticator'
- Add CryptoAuthLib template (disabled) in config.toml
* Thu Apr  1 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Make 'parsec' user part of 'tss' group to access /dev/tpm* devices
* Wed Mar 31 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 0.7.2:
  * Changelog: https://github.com/parallaxsecond/parsec/compare/0.6.0...0.7.2
* Tue Mar 30 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Recommends 'opensc' as it is used to initialize HSM keys (PKCS#11 backend)
- Add PKCS#11 template (disabled) in config.toml
- Disable Tpm backend by default in config.toml
* Fri Nov 13 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Enable 'all-providers' which adds 'pkcs11-provider' compared to
  previous config
- Drop unneeded patch:
  * pkcs11-libloading-issue.patch
* Fri Oct 23 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 0.6.0
- Update config.toml
- Add patch:
  * pkcs11-libloading-issue.patch
* Fri Oct 16 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Initiale version 0.5.0
